/*
 * gemasm.S - AES's interface to VDI (gsx)
 *
 * Copyright 2002, The EmuTOS development team
 *           1999, Caldera Thin Clients, Inc.
 *           1987, Digital Research Inc.
 *
 * This file is distributed under the GPL, version 2 or at your
 * option any later version.  See doc/license.txt for details.
 */

/*      GEMGSXIF.C      05/06/84 - 06/13/85     Lee Lorenzen            */
/*      merge High C vers. w. 2.2               8/21/87         mdf     */ 



#include "portab.h"
#include "machine.h"
#include "dos.h"
#include "obdefs.h"
#include "gsxdefs.h"
#include "funcdef.h"

#include "gemdos.h"
#include "gemrslib.h"
#include "gemgraf.h"
#include "geminput.h"
#include "gemdosif.h"
#include "gsx2.h"
#include "geminit.h"
#include "gemctrl.h"
#include "gemgsxif.h"

/*
 * Calls used in Crystal:
 *
 * vsf_interior();
 * vr_recfl();
 * vst_height();
 * vsl_type();
 * vsl_udsty();
 * g_vsl_width();
 * g_v_pline();
 * vst_clip();
 * vex_butv();
 * vex_motv();
 * vex_curv();
 * vex_timv();
 * vr_cpyfm();
 * g_v_opnwk();
 * v_clswk();
 * vq_extnd();
 * v_clsvwk( handle )
 * v_opnvwk( pwork_in, phandle, pwork_out )
 */

#define GRAFMEM         0xFFFFFFFFl


GLOBAL WORD  intout[45];
GLOBAL WORD  ptsout[12];

GLOBAL FDB   gl_tmp;

GLOBAL LONG  old_mcode;
GLOBAL LONG  old_bcode;
GLOBAL WORD  gl_moff;                /* counting semaphore   */
                                     /*  == 0 implies ON     */
                                     /*  >  0 implies OFF    */
GLOBAL LONG  gl_mlen;
GLOBAL WORD  gl_graphic;



/* Some local Prototypes: */
void  g_v_opnwk(WORD *pwork_in, WORD *phandle, WS *pwork_out );
ULONG  gsx_gbufflen(void);



ULONG gsx_mcalc()
{
    gsx_fix(&gl_tmp, 0x0L, 0, 0);           /* store screen info    */
    gl_mlen = gsx_gbufflen();
    if (gl_mlen != 0x0l)
        gl_tmp.fd_addr = GRAFMEM;             /* buffer not in sys mem */
    else
        gl_mlen =  63L * (LONG)gl_wchar * (LONG)gl_hchar * (LONG)gl_nplanes;
    return(gl_mlen);
}



void gsx_malloc()
{
    ULONG   mlen;

    mlen = gsx_mcalc();                     /* need side effects now     */
    if (gl_tmp.fd_addr != GRAFMEM)          /* buffer on graphics board? */
        gl_tmp.fd_addr = dos_alloc(mlen);     /*  no -- get from sys mem   */
}



void gsx_mfree()
{
    dos_free(gl_tmp.fd_addr);
}



void gsx_mret(LONG *pmaddr, LONG *pmlen)
{
    if (gl_tmp.fd_addr == GRAFMEM)
    {
        *pmaddr = 0x0l;
        *pmlen = 0x0l;
    }
    else
    {
        *pmaddr = gl_tmp.fd_addr;
        *pmlen = gl_mlen;
    }
}



void gsx_ncode(WORD code, WORD n, WORD m)
{
    contrl[0] = code;
    contrl[1] = n;
    contrl[3] = m;
    contrl[6] = gl_handle;
    gsx2();
}



void gsx_1code(WORD code, WORD value)
{
    intin[0] = value;
    gsx_ncode(code, 0, 1);
}



static void gsx_wsopen(void)
{
    WORD  i;

    for(i=0; i<10; i++)
        intin[i] = 1;
    intin[10] = 2;                  /* device coordinate space */
    g_v_opnwk( &intin[0], &gl_handle, &gl_ws );
    gl_graphic = TRUE;
}



void gsx_wsclose(void)
{
    gsx_ncode(CLOSE_WORKSTATION, 0, 0);
}



void ratinit(void)
{
    gsx_1code(SHOW_CUR, 0);
    gl_moff = 0;
}


void ratexit(void)
{
    gsx_moff();
}


static void gsx_setmb(void *boff, void *moff, LONG *pdrwaddr)
{
    i_lptr1( boff );
    gsx_ncode(BUT_VECX, 0, 0);
    m_lptr2( &old_bcode );

    i_lptr1( moff );
    gsx_ncode(MOT_VECX, 0, 0);
    m_lptr2( &old_mcode );

/* not used in Atari GEM:
    i_lptr1( justretf );
    gsx_ncode(CUR_VECX, 0, 0);
    m_lptr2( pdrwaddr );
*/
}



static void gsx_resetmb(void)
{
    i_lptr1( (void*)old_bcode );
    gsx_ncode(BUT_VECX, 0, 0);

    i_lptr1( (void*)old_mcode );
    gsx_ncode(MOT_VECX, 0, 0);

/* not used in Atari GEM:
    i_lptr1( (void*)drwaddr );
    gsx_ncode(CUR_VECX, 0, 0);
*/
}



void gsx_init(void)
{
    gsx_wsopen();
    gsx_start();
    gsx_setmb(far_bcha, far_mcha, &drwaddr);
    gsx_ncode(MOUSE_ST, 0, 0);
    xrat = ptsout[0];
    yrat = ptsout[1];
}



void gsx_graphic(WORD tographic)
{
    if (gl_graphic != tographic)
    {
        gl_graphic = tographic;
        if (gl_graphic)
        {
            contrl[5] = 2;
            gsx_ncode(5, 0, 0);
            gsx_setmb(far_bcha, far_mcha, &drwaddr);
        }
        else
        {
            contrl[5] = 3;
            gsx_ncode(5, 0, 0);
            gsx_resetmb();
        }
    }
}



#if !GEMDOS
void gsx_exec(LONG pcspec, WORD segenv, LONG pcmdln, LONG pfcb1, LONG pfcb2)
{
    EXEC_BLK        exec;
    LONG            lpstr;

    exec.eb_segenv = segenv;
    exec.eb_pcmdln = pcmdln;
    exec.eb_pfcb1 = pfcb1;
    exec.eb_pfcb2 = pfcb2;

    intin[0] = LLOWD(pcspec);
    intin[1] = LHIWD(pcspec);
    intin[2] = LLOWD(ADDR(&exec));
    intin[3] = LHIWD(ADDR(&exec));
    lpstr = (LONG) ADDR(rs_str(STGDOS));
    intin[4] = LLOWD( lpstr );
    intin[5] = LHIWD( lpstr );
    contrl[5] = 1;
    gsx_ncode(-1, 0, 6);
}
#endif



static void bb_set(WORD sx, WORD sy, WORD sw, WORD sh, WORD *pts1, WORD *pts2,
                   FDB *pfd, FDB *psrc, FDB *pdst)
{
    WORD            oldsx;

    /* get on word boundary */
    oldsx = sx;
    sx = (sx / 16) * 16;
    sw = ( ((oldsx - sx) + (sw + 15)) / 16 ) * 16;

    gl_tmp.fd_stand = TRUE;
    gl_tmp.fd_wdwidth = sw / 16;
    gl_tmp.fd_w = sw;
    gl_tmp.fd_h = sh;

    gsx_moff();
    pts1[0] = sx;
    pts1[1] = sy;
    pts1[2] = sx + sw - 1;
    pts1[3] = sy + sh - 1;
    pts2[0] = 0;
    pts2[1] = 0;
    pts2[2] = sw - 1;
    pts2[3] = sh - 1 ;

    gsx_fix(pfd, 0L, 0, 0);
    vro_cpyfm( S_ONLY, &ptsin[0], psrc, pdst );
    gsx_mon();
}



void bb_save(GRECT *ps)
{
    bb_set(ps->g_x, ps->g_y, ps->g_w, ps->g_h, &ptsin[0], &ptsin[4],
           &gl_src, &gl_src, &gl_tmp);
}



void bb_restore(GRECT *pr)
{
    bb_set(pr->g_x, pr->g_y, pr->g_w, pr->g_h, &ptsin[4], &ptsin[0],
           &gl_dst, &gl_tmp, &gl_dst);
}



WORD gsx_tick(void *tcode, void *ptsave)
{
    i_lptr1( tcode );
    gsx_ncode(TIM_VECX, 0, 0);
    m_lptr2( ptsave );
    return(intout[0]);
}



void gsx_mfset(LONG pmfnew)
{
    gsx_moff();
    if (!gl_ctmown)
        LWCOPY(ad_mouse, pmfnew, 37);
    LWCOPY(ad_intin, pmfnew, 37);
    gsx_ncode(ST_CUR_FORM, 0, 37);
    gsx_mon();
}



void gsx_mxmy(WORD *pmx, WORD *pmy)
{
    *pmx = xrat;
    *pmy = yrat;
}



WORD gsx_kstate()
{
    gsx_ncode(KEY_SHST, 0, 0);
    return(intout[0]);
}



void gsx_moff()
{
    if (!gl_moff)
        gsx_ncode(HIDE_CUR, 0, 0);

    gl_moff++;
}



void gsx_mon()
{
    gl_moff--;
    if (!gl_moff)
        gsx_1code(SHOW_CUR, 1);
}



WORD gsx_char()
{
    intin[0] = 4;
    intin[1] = 2;
    gsx_ncode(33, 0, 2);

    intin[0] = -1;
    intin[1] = FALSE;        /* no echo */
    gsx_ncode(31, FALSE, 2);
    if (contrl[4])
        return(intout[0]);
    else
        return(0);
}



ULONG gsx_gbufflen()
{
    gsx_1code(EXTENDED_INQUIRE, 1);
    return(LLGET(ADDR(&intout[26])));
}



/* This function was formally only called v_opnwk, but there was a
   conflict with the VDI then, so I renamed it to g_v_opnwk  - Thomas */
void g_v_opnwk(WORD *pwork_in, WORD *phandle, WS *pwork_out )
{
    WORD            *ptsptr;

    ptsptr = ((WORD *)pwork_out) + 45;
    i_ptsout( ptsptr );     /* set ptsout to work_out array */
    i_intin( pwork_in );    /* set intin to point to callers data  */
    i_intout( pwork_out );  /* set intout to point to callers data */
    gsx_ncode(OPEN_WORKSTATION, 0, 11);

    *phandle = contrl[6];
    i_intin( &intin );
    i_intout( &intout );
    i_ptsin( &ptsin );
    i_ptsout( &ptsout );
}



/* This function was formally only called v_pline, but there was a
 conflict with the VDI then, so I renamed it to g_v_pline  - Thomas */
void g_v_pline(WORD  count, WORD *pxyarray )
{
    i_ptsin( pxyarray );
    gsx_ncode(POLYLINE, count, 0);
    i_ptsin( &ptsin );
}



void vst_clip(WORD clip_flag, WORD *pxyarray )
{
    WORD            value;

    value = ( clip_flag != 0 ) ? 2 : 0;
    i_ptsin( pxyarray );
    intin[0] = clip_flag;
    gsx_ncode(TEXT_CLIP, value, 1);
    i_ptsin(&ptsin);
}


void vst_height(WORD height, WORD *pchr_width, WORD *pchr_height,
                WORD *pcell_width, WORD *pcell_height)
{
    ptsin[0] = 0;
    ptsin[1] = height;
    gsx_ncode(CHAR_HEIGHT, 1, 0);
    *pchr_width = ptsout[0];
    *pchr_height = ptsout[1];
    *pcell_width = ptsout[2];
    *pcell_height = ptsout[3];
}



void vr_recfl(WORD *pxyarray, FDB *pdesMFDB)
{
    i_ptr( pdesMFDB );
    i_ptsin( pxyarray );
    gsx_ncode(FILL_RECTANGLE, 2, 1);
    i_ptsin( &ptsin );
}



void vro_cpyfm(WORD wr_mode, WORD *pxyarray, FDB *psrcMFDB, FDB *pdesMFDB )
{
    intin[0] = wr_mode;
    i_ptr( psrcMFDB );
    i_ptr2( pdesMFDB );
    i_ptsin( pxyarray );
    gsx_ncode(COPY_RASTER_FORM, 4, 1);
    i_ptsin( &ptsin );
}



void vrt_cpyfm(WORD wr_mode, WORD *pxyarray, FDB *psrcMFDB, FDB *pdesMFDB,
               WORD fgcolor, WORD bgcolor)
{
    intin[0] = wr_mode;
    intin[1] = fgcolor;
    intin[2] = bgcolor;
    i_ptr( psrcMFDB );
    i_ptr2( pdesMFDB );
    i_ptsin( pxyarray );
    gsx_ncode(121, 4, 3);
    i_ptsin( &ptsin );
}



void vrn_trnfm(FDB *psrcMFDB, FDB *pdesMFDB)
{
    i_ptr( psrcMFDB );
    i_ptr2( pdesMFDB );

    gsx_ncode(TRANSFORM_FORM, 0, 0);
}



/*
 * This function was formally only called vsl_width, but there was a
 * conflict with the VDI then, so I renamed it to g_vsl_width  - Thomas
 */
void g_vsl_width(WORD width)
{
    ptsin[0] = width;
    ptsin[1] = 0;
    gsx_ncode(S_LINE_WIDTH, 1, 0);
}

